﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=11.2
@EndOfDesignText@
#DesignerProperty: Key: Elevation, DisplayName: Elevation, FieldType: Int, DefaultValue: 5, Description: Sets the view's elevation
#DesignerProperty: Key: NotConBackgroundColor, DisplayName: Background color when not connected, FieldType: Color, DefaultValue: 0xFFFFFFFF, Description: Sets the view's background color when not connected
#DesignerProperty: Key: NotConnectedTextColor, DisplayName: Text color when not connected, FieldType: Color, DefaultValue: 0xFF5A64F5, Description: Sets the view's text color when not connected
#DesignerProperty: Key: ConBackgroundColor, DisplayName: Background color when connected, FieldType: Color, DefaultValue: 0xFF5A64F5, Description: Sets the view's color when connected
#DesignerProperty: Key: ConnectedTextColor, DisplayName: Text color when connected, FieldType: Color, DefaultValue: 0xFFFFFFFF, Description: Sets the view's text color when connected
#DesignerProperty: Key: ErrBackgroundColor, DisplayName: Background color when there's an error, FieldType: Color, DefaultValue: 0xFFF03C3C, Description: Sets the view's background color when there's an error
#DesignerProperty: Key: ErrorTextColor, DisplayName: Text color when there's an error, FieldType: Color, DefaultValue: 0xFFFFFFFF, Description: Sets the view's text color when there's an error
#DesignerProperty: Key: ConningBackgroundColor, DisplayName: Background color when it's connecting, FieldType: Color, DefaultValue: 0xFFC3C3C3, Description: Sets the view's background color when the state is "connecting"
#DesignerProperty: Key: ConnectingTextColor, DisplayName: Text color when it's connecting, FieldType: Color, DefaultValue: 0xFF7D7D7D, Description: Sets the view's text color when the state is "connecting"

#Event: Click (SlaveAddress As Int)
#Event: ItemClick (Position As Int, Value As Object)

Sub Class_Globals
	Private mEventName As String 'ignore
	Private mCallBack As Object 'ignore
	Public mBase As B4XView
	Private xui As XUI 'ignore
	Public Tag As Object
	
	'0 = disconnesso, 1 = in connessione, 2 = connesso, 3 = errore
	Public Const STATE_NOT_CONNECTED As Int = 0
	Public Const STATE_CONNECTING As Int = 1
	Public Const STATE_CONNECTED As Int = 2
	Public Const STATE_ERROR As Int = 3
	
	Private mStateConnected, mStateNotConnected, mStateConnecting, mStateError As State
	
	'Variables
	Private currentState As Int
	Private cornRad As Int
	Private dropDownSpinnerTextColor As Int
	Private mSlaveAddress As Int
	Private changingState As Boolean = False
	
	'Properties
	Private baseElev As Int
	Private connectedColor, notConnectedColor, errColor, connectingColor As Int
	Private drwConnected, drwNotConnected, drwError, drwConnecting As ColorDrawable
	Private txtColorConnected, txtColorNotConnected, txtColorError, txtColorConnecting As Int
	
	'Views
	Private mBasePanel As Panel
	Private mStateLabel As Label
	Private mBackgroundPanel(2) As Panel
	Private mSpin As Spinner
End Sub

Public Sub Initialize (Callback As Object, EventName As String)
	mEventName = EventName
	mCallBack = Callback
	
	If Not(mStateLabel.IsInitialized) Then mStateLabel.Initialize("")
	If Not(mBasePanel.IsInitialized) Then mBasePanel.Initialize("")
	For Each pan As Panel In mBackgroundPanel
		If Not(pan.IsInitialized) Then pan.Initialize("")
	Next
	If Not(mSpin.IsInitialized) Then mSpin.Initialize("mSpin")
	
	For i = 0 To 3
		Dim states() As State = Array As State(mStateNotConnected, mStateConnecting, mStateConnected, mStateError)
		states(i).State = i
		Select i
			Case 0
				states(i).Text = "Non connesso"
			Case 1
				states(i).Text = "In connessione..."
			Case 2
				states(i).Text = "Connesso"
			Case 3
				states(i).Text = "Errore"
		End Select
	Next
End Sub

'Base type must be Object
Public Sub DesignerCreateView (Base As Object, Lbl As Label, Props As Map)
	mBase = Base
	Tag = mBase.Tag
	mBase.Tag = Me
	
	'Setta al massimo il raggio degli angoli per essere più arrotondato possibile
	cornRad = 360
	mBase.SetColorAndBorder(0xFFFFFFFF, 0, 0xFFFFFFFF, cornRad)
	
	'Raccoglie le proprietà definite nel designer e le salva nelle variabili locali
	baseElev = Props.Get("Elevation")
	connectedColor = Props.Get("ConBackgroundColor")
	notConnectedColor = Props.Get("NotConBackgroundColor")
	errColor = Props.Get("ErrBackgroundColor")
	connectingColor = Props.Get("ConningBackgroundColor")
	txtColorConnected = Props.Get("ConnectedTextColor")
	txtColorNotConnected = Props.Get("NotConnectedTextColor")
	txtColorError = Props.Get("ErrorTextColor")
	txtColorConnecting = Props.Get("ConnectingTextColor")
	
	'Inizializzo i drawable per ogni stato con i colori appena salvati
	drwConnected.Initialize(connectedColor, cornRad)
	drwNotConnected.Initialize(notConnectedColor, cornRad)
	drwError.Initialize(errColor, cornRad)
	drwConnecting.Initialize(connectingColor, cornRad)
	
	'Setto come stato di default il NonConnesso nel primo pannello
	mBackgroundPanel(0).Background = drwNotConnected
	'Imposto il Visible del primo pannello a True mentre quello dell'altro a False
	mBackgroundPanel(0).Visible = True
	mBackgroundPanel(1).Visible = False
	
	'Imposto lo spinner
	Dim lst As List
	lst.Initialize
	For i = 1 To 247
		lst.Add(i)
	Next
	mSpin.AddAll(lst)
	mSpin.SelectedIndex = mSpin.Size - 1
	dropDownSpinnerTextColor = 0xFF212121
	
'	'Setta i simboli per ogni stato
	mStateNotConnected.StateSymbol = Chr(0xF127)
	mStateConnecting.StateSymbol = Chr(0xF192)
	mStateConnected.StateSymbol = Chr(0xF00C)
	mStateError.StateSymbol = Chr(0xF12A)
	
	'Setta il colore dello sfondo e il colore del testo per ogni stato
	mStateNotConnected.MainColor = drwNotConnected
	mStateNotConnected.TextColor = txtColorNotConnected
	mStateConnecting.MainColor = drwConnecting
	mStateConnecting.TextColor = txtColorConnecting
	mStateConnected.MainColor = drwConnected
	mStateConnected.TextColor = txtColorConnected
	mStateError.MainColor = drwError
	mStateError.TextColor = txtColorError
	
	'Prende le informazioni da Lbl e le setta in mStateLabel
	mStateLabel.TextSize = Lbl.TextSize
	mStateLabel.TextColor = Lbl.TextColor
	mStateLabel.Typeface = Lbl.Typeface
	mStateLabel.Gravity = Lbl.Gravity
	mStateLabel.SingleLine = True
	mStateLabel.Text = ""
	
	'Aggiunge tutte le view a mBase senza settarne la posizione
	For Each pan As Panel In mBackgroundPanel
		mBase.AddView(pan, 0, 0, 0, 0)
	Next
	mBase.AddView(mBasePanel, 0, 0, 0, 0)
	mBasePanel.AddView(mStateLabel, 0, 0, 0, 0)
	mBasePanel.AddView(mSpin, 0, 0, 0, 0)
	
	Dim drw As ColorDrawable
	drw.Initialize(Colors.Transparent, cornRad)
	mBasePanel.Background = drw
	
	currentState = STATE_CONNECTED
	mSlaveAddress = 247
	
	Base_Resize(mBase.Width, mBase.Height)
	UpdateTextsAndColors(STATE_NOT_CONNECTED)
End Sub

Private Sub Base_Resize (Width As Double, Height As Double)
	If Width <= 48dip Then Width = 55dip
	If Height < 32dip Then Height = 32dip
	Dim pan As Panel = mBase
	pan.SetElevationAnimated(0, baseElev)
	
	For Each pan As Panel In mBackgroundPanel
		pan.SetLayoutAnimated(0, 0, 0, Width, Height)
	Next
	mBasePanel.SetLayoutAnimated(0, 0, 0, Width, Height)
	mSpin.SetLayoutAnimated(0, Height * 1.125, Height * 0.0625, Width - (Height * 1.5), Height * 0.5625)
	mStateLabel.SetLayoutAnimated(0, Height * 1.125, Height * 0.625, Width - (Height * 1.5), Height * 0.3125)
End Sub

Private Sub mSpin_ItemClick (Position As Int, Value As Object)
	If SubExists(mCallBack, mEventName & "_ItemClick") Then CallSubDelayed3(mCallBack, mEventName & "_ItemClick", Position, Value)
End Sub

Private Sub OpenSpinner
	Dim r As Reflector
	r.Target = mSpin
	r.RunMethod("performClick")
End Sub

'Aggiorna lo stato della View con quello passato come argomento
'coerentemente con i colori di sfondo, testo e immagini impostate all'inizio
Private Sub UpdateTextsAndColors (ToState As Int)
	If ToState < 0 Or ToState > 4 Then Return
	changingState = True
	Dim newState As State
	Select ToState
		Case mStateConnected.State
			newState = mStateConnected
		Case mStateConnecting.State
			newState = mStateConnecting
		Case mStateError.State
			newState = mStateError
		Case mStateNotConnected.State
			newState = mStateNotConnected
	End Select
	
	Dim pnlVisible, pnlNotVisible As Panel
	If mBackgroundPanel(0).Visible Then
		pnlVisible = mBackgroundPanel(0)
		pnlNotVisible = mBackgroundPanel(1)
	Else
		pnlVisible = mBackgroundPanel(1)
		pnlNotVisible = mBackgroundPanel(0)
	End If
	pnlNotVisible.Background = newState.MainColor
	
	mSpin.TextColor = newState.TextColor
	mSpin.DropdownTextColor = dropDownSpinnerTextColor
	mStateLabel.TextColor = newState.TextColor
	mStateLabel.Text = newState.Text
	RefreshSpinner
	
	pnlVisible.SetVisibleAnimated(100, False)
	pnlNotVisible.SetVisibleAnimated(100, True)
	Sleep(100)
	
	currentState = newState.State
	
	changingState = False
End Sub

'Per fare in modo di aggiornare i colori del testo e del menu a tendina dello spinner
'bisogna cancellarlo e ripopolarlo ogni volta (operazione molto veloce)
'https://www.b4x.com/android/forum/threads/setting-spinner-color-at-runtime.28435/post-183368
Private Sub RefreshSpinner
	Dim arraytemp(mSpin.Size) As String
	Dim selectedIndex As Int
	For x = 0 To mSpin.Size - 1
		arraytemp(x) = mSpin.GetItem(x)
	Next
	selectedIndex = mSpin.selectedIndex
	mSpin.Clear
	
'	'Aggiorna colore freccia spinner
'	Dim jo As JavaObject = mSpin.Background
'	jo.RunMethod("setColorFilter", Array(mSpin.TextColor, "SRC_ATOP"))
	
	Dim jo1 As JavaObject = mSpin.Background
	Dim p As Phone
	If p.SdkVersion < 29 Then
		jo1.RunMethod("setColorFilter", Array(mSpin.TextColor, "SRC_ATOP"))
	Else
		Dim jo2 As JavaObject
		jo2 = jo2.InitializeNewInstance("android.graphics.BlendModeColorFilter", Array(mSpin.TextColor, "SRC_ATOP"))
		jo1.RunMethod("setColorFilter", Array(jo2))
	End If
	
	For x = 0 To arraytemp.Length - 1
		mSpin.Add(arraytemp(x))
	Next
	mSpin.selectedIndex = selectedIndex
End Sub

#Region Properties
Public Sub getHeight As Int
	Return mBase.Height
End Sub

Public Sub setHeight(Height As Int)
	mBase.Height = Height
	Base_Resize(mBase.Width, mBase.Height)
End Sub

Public Sub getWidth As Int
	Return mBase.Width
End Sub

Public Sub setWidth(Width As Int)
	mBase.Width = Width
	Base_Resize(mBase.Width, mBase.Height)
End Sub

Public Sub getTop As Int
	Return mBase.Top
End Sub

Public Sub setTop(Top As Int)
	mBase.Top = Top
	Base_Resize(mBase.Width, mBase.Height)
End Sub

Public Sub getLeft As Int
	Return mBase.Left
End Sub

Public Sub setLeft(Left As Int)
	mBase.Left = Left
	Base_Resize(mBase.Width, mBase.Height)
End Sub

Public Sub getSpinner As Spinner
	Return mSpin
End Sub

Public Sub getLabel As Label
	Return mStateLabel
End Sub

Public Sub setSlaveAddress(NewVal As Int)
	If NewVal = mSlaveAddress Then Return
	mSlaveAddress = NewVal
	mSpin.SelectedIndex = NewVal - 1
	setState(STATE_NOT_CONNECTED)
End Sub

Public Sub getSlaveAddress As Int
	Return mSlaveAddress
End Sub

Public Sub setState(NewState As Int)
	UpdateTextsAndColors(NewState)
End Sub

Public Sub getState As Int
	Return currentState
End Sub

Public Sub setVisible(vis As Boolean)
	mBase.Visible = vis
End Sub

Public Sub getVisible As Boolean
	Return mBase.Visible
End Sub
#End Region

#Region Methods
Public Sub SetLayout(Left As Int, Top As Int, Width As Int, Height As Int)
	mBase.SetLayoutAnimated(0, Left, Top, Width, Height)
	Base_Resize(mBase.Width, mBase.Height)
End Sub

Public Sub SetLayoutAnimated(Duration As Int, Left As Int, Top As Int, Width As Int, Height As Int)
	Dim start As Long = DateTime.Now
	Do While DateTime.Now < start + Duration
		Dim p As Float = (DateTime.Now - start) / Duration
		SetLayout(mBase.Left + ((Left - mBase.Left) * p), mBase.Top + ((Top - mBase.Top) * p), mBase.Width + ((Width - mBase.Width) * p), mBase.Height + ((Height - mBase.Height) * p))
		Sleep(10)
	Loop
End Sub
#End Region